@@ -1,14 +1,17 @@
 """Template Tags for DumpDie"""
 
+# System Imports.
 import inspect
 import re
+import traceback
 import types
-
 from decimal import Decimal
 
+# Third-Party Imports.
 from django import template
 from django.forms.boundfield import BoundField
 
+# Internal Imports.
 from django_dump_die.constants import (
     SIMPLE_TYPES,
     INTERMEDIATE_TYPES,
@@ -26,7 +29,6 @@ from django_dump_die.constants import (
     INCLUDE_FUNCTIONS,
     PYTZ_PRESENT,
 )
-
 from django_dump_die.utils import (
     get_dumped_object_info,
     generate_unique_from_obj,
@@ -560,13 +562,24 @@ def get_obj_values(obj):
     functions = []      # (attr, doc, access_modifier)
 
     try:
-
         # Attempt to get member values of object. Falls back to empty list on failure.
         members = get_members(obj)
 
     except Exception as exception:
         # On exception, add exception to attributes and return the attributes.
-        attributes.append(['EXCEPTION', str(exception), None, 'empty', 'Exception'])
+
+        # First get exception data.
+        tb = exception.__traceback__
+        title = 'Exception Occurred\n\n'
+        title += f'Exception Type: {type(exception).__name__}\n'
+        title += f'Exception Value: {str(exception)}\n\n'
+        title += 'Traceback (most recent call last):\n\n'
+        for entry in traceback.format_tb(tb):
+            title += f'{entry}\n'
+        title += 'End of traceback'
+
+        # Add exception data to attributes and return.
+        attributes.append(['EXCEPTION', str(exception), None, 'empty', title])
         return (attributes, functions)
 
     # Once all members have been collected, attempt to figure out what type, access modifier, css class, and title
@@ -643,7 +656,18 @@ def get_obj_values(obj):
                 attributes.append([attr, value, access_modifier, css_class, title])
 
         except Exception as inner_exception:
-            attributes.append(['EXCEPTION', str(inner_exception), None, 'empty', 'Exception'])
+            # First get exception data.
+            tb = inner_exception.__traceback__
+            title = 'Exception:\n'
+            while tb is not None:
+                line_num = tb.tb_lineno
+                name = tb.tb_frame.f_code.co_name
+                filename = tb.tb_frame.f_code.co_filename
+                tb = tb.tb_next
+                title += '\n[ {0} : {1} ] {2}\n'.format(line_num, name, filename)
+
+            # Add exception data to attributes and return.
+            attributes.append(['EXCEPTION', str(inner_exception), None, 'empty', title])
 
     # Attempt to sort the functions and just ignore any errors.
     try:
