@@ -5,19 +5,28 @@ Helps prevent package errors relating to example view load from propagating
 to general package usage.
 """
 
-from enum import Enum
+# System Imports.
+import copy
 import os
 from datetime import datetime, timedelta
 from decimal import Decimal
-from django.core.files import File
-from django.db import models
-from django.forms import ModelForm
-from django.utils import timezone
+from enum import Enum
 from pathlib import Path, PosixPath, PurePath, WindowsPath
 from types import ModuleType
 
+# Third-Party Imports.
+from django.core.files import File
+from django.db import models
+from django.forms import ModelForm
+from django.http import QueryDict
+from django.shortcuts import render
+from django.template.response import TemplateResponse
+from django.utils import timezone
+
+# Internal Imports.
 from django_dump_die.constants import PYTZ_PRESENT, ZONEINFO_PRESENT
 
+# Imports that may not be accessible, depending on local python environment setup.
 if PYTZ_PRESENT:
     import pytz
 if ZONEINFO_PRESENT:
@@ -424,21 +433,10 @@ def dump_intermediate_types():
         dump(sample_windows_path)
 
 
-def dump_complex_types():
+class dump_complex_types():
     """Dump Complex Types"""
-    # Generate variables to dump.
-    sample_set = {'A', 'B', 'C'}
-    sample_frozen_set = frozenset({'D', 'E', 'F'})
-    sample_tuple = ('A', 12, True)
-    sample_list = ['A', 12, True]
-    sample_dict = {
-        'first': 'A',
-        'second': 12,
-        'third': True,
-    }
-    sample_memory_view = memoryview(bytearray([8, 9, 10, 11]))
 
-    sample_complex_set = {
+    sample_multilevel_set = {
         (
             'A',
             12,
@@ -451,7 +449,7 @@ def dump_complex_types():
         ),
     }
 
-    sample_complex_tuple = (
+    sample_multilevel_tuple = (
         {
             'first': 'A',
             'second': 12,
@@ -464,7 +462,7 @@ def dump_complex_types():
         },
     )
 
-    sample_complex_list = [
+    sample_multilevel_list = [
         {
             'first': 'A',
             'second': 12,
@@ -477,7 +475,7 @@ def dump_complex_types():
         },
     ]
 
-    sample_complex_dict = {
+    sample_multilevel_dict = {
         'initial': {
             'first': 'A',
             'second': 12,
@@ -490,32 +488,126 @@ def dump_complex_types():
         },
     }
 
-    # Call dump on all generated variables.
-    dump('')
-    dump('Basic object examples:')
-    dump(sample_set)
-    dump(sample_frozen_set)
-    dump(sample_tuple)
-    dump(sample_list)
-    dump(sample_dict)
-    dump(sample_memory_view)
+    def dump_set(self):
+        # Generate variables to dump.
+        sample_set = {'A', 'B', 'C'}
 
-    dump('')
-    dump('Elaborate object examples:')
-    dump(sample_complex_set)
-    dump(sample_complex_tuple)
-    dump(sample_complex_list)
-    dump(sample_complex_dict)
-    dump(SampleEnum)
+        # Dump variable.
+        dump(sample_set)
 
-    dump('')
-    dump('Examples of pulling items/indexes/subsets from above objects:')
-    dump(sample_complex_list[0])
-    dump(sample_complex_tuple[0])
-    dump(sample_complex_tuple[0].items)
-    dump(sample_complex_dict['initial'])
-    dump(SampleEnum.RED)
-    dump(SampleEnum.BLUE)
+    def dump_frozen_set(self):
+        # Generate variables to dump.
+        sample_frozen_set = frozenset({'D', 'E', 'F'})
+
+        # Dump variable.
+        dump(sample_frozen_set)
+
+    def dump_tuple(self):
+        # Generate variables to dump.
+        sample_tuple = ('A', 12, True)
+
+        # Dump variable.
+        dump(sample_tuple)
+
+    def dump_list(self):
+        # Generate variables to dump.
+        sample_list = ['A', 12, True]
+
+        # Dump variable.
+        dump(sample_list)
+
+    def dump_dict(self):
+        # Generate variables to dump.
+        sample_dict = {
+            'first': 'A',
+            'second': 12,
+            'third': True,
+        }
+
+        # Dump variable.
+        dump(sample_dict)
+
+    def dump_querydict(self):
+        # Generate variables to dump.
+        sample_querydict = QueryDict('first="A"&second=12&third=True&first="B"&first="C"')
+
+        # Dump variable.
+        dump(sample_querydict)
+
+    def dump_enum(self):
+        # Dump variable.
+        dump(SampleEnum)
+
+    def dump_memory_view(self):
+        # Generate variables to dump.
+        sample_memory_view = memoryview(bytearray([8, 9, 10, 11]))
+
+        # Dump variable.
+        dump(sample_memory_view)
+
+    def dump_multilevel_set(self):
+        # Dump variable.
+        dump(self.sample_multilevel_set)
+
+    def dump_multilevel_tuple(self):
+        # Dump variable.
+        dump(self.sample_multilevel_tuple)
+
+    def dump_multilevel_list(self):
+        dump(self.sample_multilevel_list)
+
+    def dump_multilevel_dict(self):
+        # Dump variable.
+        dump(self.sample_multilevel_dict)
+
+    def dump_list_subitem(self):
+        # Dump variable.
+        dump(self.sample_multilevel_list[0])
+
+    def dump_tuple_subitem(self):
+        # Dump variable.
+        dump(self.sample_multilevel_tuple[0])
+
+    def dump_tuple_subitem_function(self):
+        # Dump variable.
+        dump(self.sample_multilevel_tuple[0].items)
+
+    def dump_dict_subitem(self):
+        # Dump variable.
+        dump(self.sample_multilevel_dict['initial'])
+
+    def dump_enum_subitem(self):
+        # Dump variable.
+        dump(SampleEnum.RED)
+        dump(SampleEnum.BLUE)
+
+    def dump_all_objects(self):
+        # Call dump on all generated variables.
+        dump('')
+        dump('Minimal object examples:')
+        self.dump_set()
+        self.dump_frozen_set()
+        self.dump_tuple()
+        self.dump_list()
+        self.dump_dict()
+        self.dump_querydict()
+        self.dump_memory_view()
+        self.dump_enum()
+
+        dump('')
+        dump('Nested object examples:')
+        self.dump_multilevel_set()
+        self.dump_multilevel_tuple()
+        self.dump_multilevel_list()
+        self.dump_multilevel_dict()
+
+        dump('')
+        dump('Examples of pulling items/indexes/subsets from above objects:')
+        self.dump_list_subitem()
+        self.dump_tuple_subitem()
+        self.dump_tuple_subitem_function()
+        self.dump_dict_subitem()
+        self.dump_enum_subitem()
 
 
 def dump_function_types():
@@ -635,7 +727,16 @@ def dump_model_types():
     # Call dump on all generated variables.
     dump('')
     dump('Model and ModelForm class examples:')
-    dump(SampleDjangoModel)
+    # TODO: See if there is a better way to "fix" this.
+    # Need to remove the Many-To-Many relationship when dumping the class
+    # definition (non-instance) because python's get_members can't handle a
+    # m2m when the "object" does not have an id. Which it can't due to it being
+    # a class definition and not an instance. Problem is solved when an instance
+    # is created. See below dump of instance.
+    SampleDjangoModelNoManyRelation = copy.deepcopy(SampleDjangoModel)
+    delattr(SampleDjangoModelNoManyRelation, 'sample_many')
+    dump(SampleDjangoModelNoManyRelation)
+
     dump(SampleModelForm)
 
     dump('')
@@ -716,6 +817,43 @@ def dump_syspath_types():
         dump(windows_path)
 
 
+def dump_django_request_response_cycle_types(request):
+    """Dump Django request-response cycle Types."""
+
+    # Generate variables to dump.
+    sample_query_dict = QueryDict('one_val=test&two_vals=one&two_vals=2')
+    sample_query_dict = copy.deepcopy(sample_query_dict)
+    sample_query_dict.appendlist('example_field_list', 'username')
+    sample_query_dict.appendlist('example_field_list', 'first_name')
+    sample_query_dict.appendlist('example_field_list', 'last_name')
+    sample_query_dict.appendlist('example_types', None)
+    sample_query_dict.appendlist('example_types', True)
+    sample_query_dict.appendlist('example_types', 5)
+    sample_query_dict.appendlist('example_types', 3.0)
+
+    sample_request = request
+    sample_http_response = render(request, 'django_dump_die/sample.html', {})
+    sample_template_response = TemplateResponse(request, 'django_dump_die/sample.html', {})
+    sample_template_response.render()
+
+    # Call dump on generated variables.
+    dump('')
+    dump('QueryDict object (GET and POST are instances of this):')
+    dump(sample_query_dict)
+
+    dump('')
+    dump('Request object:')
+    dump(sample_request)
+
+    dump('')
+    dump('HttpResponse object:')
+    dump(sample_http_response)
+
+    dump('')
+    dump('TemplateResponse object:')
+    dump(sample_template_response)
+
+
 def dump_edgecase_types():
     """Dump Edge Case Types"""
     # Call dump on problem children.
